@@ -3,15 +3,17 @@
 
 #include "shellhandler.h"
 
+#include "common/treelandlogging.h"
 #include "core/qmlengine.h"
-#include "popupsurfacecontainer.h"
+#include "core/windowsizestore.h"
 #include "layersurfacecontainer.h"
+#include "modules/app-id-resolver/appidresolver.h"
 #include "modules/dde-shell/ddeshellmanagerinterfacev1.h"
 #include "rootsurfacecontainer.h"
 #include "seat/helper.h"
 #include "surface/surfacewrapper.h"
+#include "treelandconfig.hpp"
 #include "workspace/workspace.h"
-#include "common/treelandlogging.h"
 
 #include <xcb/xcb.h>
 
@@ -31,6 +33,12 @@
 #include <qwcompositor.h>
 #include <qwxwaylandsurface.h>
 
+#include <QPointer>
+#include <QTimer>
+
+#include <optional>
+#include <algorithm>
+
 QW_USE_NAMESPACE
 WAYLIB_SERVER_USE_NAMESPACE
 
@@ -44,6 +52,7 @@ ShellHandler::ShellHandler(RootSurfaceContainer *rootContainer)
     , m_topContainer(new LayerSurfaceContainer(rootContainer))
     , m_overlayContainer(new LayerSurfaceContainer(rootContainer))
     , m_popupContainer(new SurfaceContainer(rootContainer))
+    , m_windowSizeStore(new WindowSizeStore(this))
 {
     m_backgroundContainer->setZ(RootSurfaceContainer::BackgroundZOrder);
     m_bottomContainer->setZ(RootSurfaceContainer::BottomZOrder);
@@ -53,6 +62,95 @@ ShellHandler::ShellHandler(RootSurfaceContainer *rootContainer)
     m_popupContainer->setZ(RootSurfaceContainer::PopupZOrder);
 }
 
+void ShellHandler::updateWrapperContainer(SurfaceWrapper *wrapper, WSurface *parentSurface)
+{
+    if (wrapper->parentSurface())
+        wrapper->parentSurface()->removeSubSurface(wrapper);
+
+    auto oldContainer = wrapper->container();
+    if (parentSurface) {
+        auto parentWrapper = m_rootSurfaceContainer->getSurface(parentSurface);
+        auto parentContainer = qobject_cast<SurfaceContainer *>(parentWrapper->container());
+        parentWrapper->addSubSurface(wrapper);
+        if (oldContainer != parentContainer) {
+            if (oldContainer)
+                oldContainer->removeSurface(wrapper);
+            if (auto ws = qobject_cast<Workspace *>(parentContainer))
+                ws->addSurface(wrapper, parentWrapper->workspaceId());
+            else
+                parentContainer->addSurface(wrapper);
+        }
+    } else {
+        if (oldContainer) {
+            if (qobject_cast<Workspace *>(oldContainer) == nullptr) {
+                oldContainer->removeSurface(wrapper);
+                m_workspace->addSurface(wrapper);
+            }
+            // else do nothing, already in workspace
+        } else {
+            m_workspace->addSurface(wrapper);
+        }
+    }
+}
+
+// Prelaunch splash request: create a SurfaceWrapper that is not yet bound to a shellSurface
+void ShellHandler::handlePrelaunchSplashRequested(const QString &appId,
+                                                  QW_NAMESPACE::qw_buffer *iconBuffer)
+{
+    if (!Helper::instance()->globalConfig()->enablePrelaunchSplash())
+        return;
+    if (!m_appIdResolverManager)
+        return;
+    if (appId.isEmpty())
+        return;
+    // If a prelaunch wrapper with the same appId already exists, skip creating a duplicate.
+    if (std::any_of(m_prelaunchWrappers.cbegin(),
+                    m_prelaunchWrappers.cend(),
+                    [&appId](SurfaceWrapper *w) { return w && w->appId() == appId; })) {
+        return;
+    }
+    QSize initialSize;
+    if (m_windowSizeStore) {
+        const QSize last = m_windowSizeStore->lastSizeFor(appId);
+        if (last.isValid() && last.width() > 0 && last.height() > 0) {
+            initialSize = last;
+        }
+    }
+    auto *wrapper = new SurfaceWrapper(Helper::instance()->qmlEngine(),
+                                       nullptr,
+                                       initialSize,
+                                       appId,
+                                       iconBuffer);
+    m_workspace->addSurface(wrapper);
+    m_prelaunchWrappers.append(wrapper);
+
+    // After configurable timeout, if still unmatched (not converted and still in the list), destroy
+    // the splash wrapper
+    qlonglong timeoutMs = Helper::instance()->globalConfig()->prelaunchSplashTimeoutMs();
+    // Bounds: <=0 disables auto-destroy, >60000 clamps to 60000
+    if (timeoutMs > 60000) {
+        timeoutMs = 60000;
+    }
+    if (timeoutMs > 0) {
+        QTimer::singleShot(static_cast<int>(timeoutMs),
+                           wrapper,
+                           [this, wrapper = QPointer<SurfaceWrapper>(wrapper)] {
+                               if (!wrapper) {
+                                   return; // Wrapper already destroyed elsewhere
+                               }
+                               int idx = m_prelaunchWrappers.indexOf(wrapper);
+                               if (idx < 0) {
+                                   return; // Already matched or removed earlier
+                               }
+                               qCDebug(treelandShell)
+                                   << "Prelaunch splash timeout, destroy wrapper appId="
+                                   << wrapper->appId();
+                               m_prelaunchWrappers.removeAt(idx);
+                               m_rootSurfaceContainer->destroyForSurface(wrapper);
+                           });
+    }
+}
+
 Workspace *ShellHandler::workspace() const
 {
     return m_workspace;
@@ -100,23 +198,11 @@ WXWayland *ShellHandler::createXWayland(WServer *server,
     m_xwaylands.append(xwayland);
     xwayland->setSeat(seat);
     connect(xwayland, &WXWayland::surfaceAdded, this, &ShellHandler::onXWaylandSurfaceAdded);
-    connect(xwayland, &WXWayland::ready, xwayland, [xwayland, this] {
+    connect(xwayland, &WXWayland::ready, xwayland, [xwayland] {
         auto atomPid = xwayland->atom("_NET_WM_PID");
         xwayland->setAtomSupported(atomPid, true);
         auto atomNoTitlebar = xwayland->atom("_DEEPIN_NO_TITLEBAR");
         xwayland->setAtomSupported(atomNoTitlebar, true);
-        // TODO: set other xsettings and sync
-        setResourceManagerAtom(
-            xwayland,
-            QString("Xft.dpi:\t%1")
-                .arg(96 * m_rootSurfaceContainer->window()->effectiveDevicePixelRatio())
-                .toUtf8());
-        connect(Helper::instance()->window(),
-                &WOutputRenderWindow::effectiveDevicePixelRatioChanged,
-                xwayland,
-                [xwayland, this](qreal dpr) {
-                    setResourceManagerAtom(xwayland, QString("Xft.dpi:\t%1").arg(96 * dpr).toUtf8());
-                });
     });
     return xwayland;
 }
@@ -124,7 +210,10 @@ WXWayland *ShellHandler::createXWayland(WServer *server,
 void ShellHandler::removeXWayland(WXWayland *xwayland)
 {
     m_xwaylands.removeOne(xwayland);
-    xwayland->safeDeleteLater();
+    // Ensure xwayland is removed from interfaceList
+    if (auto server = WServer::from(xwayland))
+        server->detach(xwayland);
+    delete xwayland;
 }
 
 void ShellHandler::initInputMethodHelper(WServer *server, WSeat *seat)
@@ -144,51 +233,117 @@ void ShellHandler::initInputMethodHelper(WServer *server, WSeat *seat)
 
 void ShellHandler::onXdgToplevelSurfaceAdded(WXdgToplevelSurface *surface)
 {
-    auto wrapper = new SurfaceWrapper(Helper::instance()->qmlEngine(),
-                                      surface,
-                                      SurfaceWrapper::Type::XdgToplevel);
+    // If there are prelaunch wrappers and the resolver is available -> attempt async resolve;
+    // remaining logic continues in the callback on success
+    if (!m_prelaunchWrappers.isEmpty() && m_appIdResolverManager) {
+        int pidfd = surface->pidFD();
+        if (pidfd >= 0) {
+            // Register pending before starting async resolve (unified list)
+            m_pendingAppIdResolveToplevels.append(surface);
 
+            bool started = m_appIdResolverManager->resolvePidfd(
+                pidfd,
+                [this, surface = QPointer<WXdgToplevelSurface>(surface)](const QString &appId) {
+                    auto raw = surface.data();
+                    if (!raw)
+                        return; // surface was deleted before callback
+                    int idx = m_pendingAppIdResolveToplevels.indexOf(raw);
+                    if (idx < 0)
+                        return; // removed before callback
+                    SurfaceWrapper *w = matchOrCreateXdgWrapper(raw, appId);
+                    initXdgWrapperCommon(raw, w);
+                    m_pendingAppIdResolveToplevels.removeAt(idx);
+                });
+            if (started) {
+                qCDebug(treelandShell) << "AppIdResolver request sent (callback) pidfd=" << pidfd;
+                return; // async path handles creation
+            } else {
+                int idx = m_pendingAppIdResolveToplevels.indexOf(surface);
+                if (idx >= 0)
+                    m_pendingAppIdResolveToplevels.removeAt(idx);
+                qCDebug(treelandShell)
+                    << "AppIdResolverManager present but requestResolve failed pidfd=" << pidfd;
+            }
+        }
+    }
+    // Async resolve not started or failed -> directly match or create
+    SurfaceWrapper *wrapper = matchOrCreateXdgWrapper(surface, QString());
+    initXdgWrapperCommon(surface, wrapper);
+}
+
+SurfaceWrapper *ShellHandler::matchOrCreateXdgWrapper(WXdgToplevelSurface *surface,
+                                                      const QString &targetAppId)
+{
+    SurfaceWrapper *wrapper = nullptr;
+
+    if (!targetAppId.isEmpty()) {
+        for (int i = 0; i < m_prelaunchWrappers.size(); ++i) {
+            auto *candidate = m_prelaunchWrappers[i];
+            if (candidate->appId() == targetAppId) {
+                qCDebug(treelandShell) << "match prelaunch xdg" << targetAppId;
+                m_prelaunchWrappers.removeAt(i);
+                wrapper = candidate;
+                wrapper->convertToNormalSurface(surface, SurfaceWrapper::Type::XdgToplevel);
+                break;
+            }
+        }
+    }
+    if (!wrapper) {
+        wrapper = new SurfaceWrapper(Helper::instance()->qmlEngine(),
+                                     surface,
+                                     SurfaceWrapper::Type::XdgToplevel,
+                                     targetAppId);
+        m_workspace->addSurface(wrapper);
+    }
+    return wrapper;
+}
+
+void ShellHandler::initXdgWrapperCommon(WXdgToplevelSurface *surface, SurfaceWrapper *wrapper)
+{
     if (DDEShellSurfaceInterface::get(surface->surface())) {
         handleDdeShellSurfaceAdded(surface->surface(), wrapper);
     }
-
     auto updateSurfaceWithParentContainer = [this, wrapper, surface] {
-        if (wrapper->parentSurface())
-            wrapper->parentSurface()->removeSubSurface(wrapper);
-        if (wrapper->container())
-            wrapper->container()->removeSurface(wrapper);
-
-        if (auto parent = surface->parentSurface()) {
-            auto parentWrapper = m_rootSurfaceContainer->getSurface(parent);
-            auto container = qobject_cast<SurfaceContainer *>(parentWrapper->container());
-            parentWrapper->addSubSurface(wrapper);
-            if (auto workspace = qobject_cast<Workspace *>(container))
-                workspace->addSurface(wrapper, parentWrapper->workspaceId());
-            else
-                container->addSurface(wrapper);
-        } else {
-            m_workspace->addSurface(wrapper);
-        }
+        updateWrapperContainer(wrapper, surface->parentSurface());
     };
 
     surface->safeConnect(&WXdgToplevelSurface::parentXdgSurfaceChanged,
                          this,
                          updateSurfaceWithParentContainer);
     updateSurfaceWithParentContainer();
+    Q_ASSERT(wrapper->parentItem());
     setupSurfaceWindowMenu(wrapper);
     setupSurfaceActiveWatcher(wrapper);
-
-    Q_ASSERT(wrapper->parentItem());
     Q_EMIT surfaceWrapperAdded(wrapper);
 }
 
 void ShellHandler::onXdgToplevelSurfaceRemoved(WXdgToplevelSurface *surface)
 {
     auto wrapper = m_rootSurfaceContainer->getSurface(surface);
+    // If async resolve still pending, cancel it. If wrapper never created, just return: compositor
+    // never exposed this surface (from treeland's perspective).
+    if (!wrapper) {
+        if (!m_pendingAppIdResolveToplevels.removeOne(surface)) {
+            qCWarning(treelandShell)
+                << "onXdgToplevelSurfaceRemoved for unknown surface" << surface;
+        }
+        return;
+    }
     auto interface = DDEShellSurfaceInterface::get(surface->surface());
     if (interface) {
         delete interface;
     }
+    // Persist the last size of a normal window (prefer normalGeometry) when an appId is present
+    if (m_windowSizeStore && !wrapper->appId().isEmpty()) {
+        QSizeF sz = wrapper->normalGeometry().size();
+        if (!sz.isValid() || sz.isEmpty()) {
+            sz = wrapper->geometry().size();
+        }
+        const QSize s = sz.toSize();
+        if (s.isValid() && s.width() > 0 && s.height() > 0) {
+            m_windowSizeStore->saveSize(wrapper->appId(), s);
+        }
+    }
     Q_EMIT surfaceWrapperAboutToRemove(wrapper);
     m_rootSurfaceContainer->destroyForSurface(wrapper);
 }
@@ -238,49 +393,113 @@ void ShellHandler::onXdgPopupSurfaceRemoved(WXdgPopupSurface *surface)
 
 void ShellHandler::onXWaylandSurfaceAdded(WXWaylandSurface *surface)
 {
-    surface->safeConnect(&qw_xwayland_surface::notify_associate, this, [this, surface] {
-        auto wrapper = new SurfaceWrapper(Helper::instance()->qmlEngine(),
-                                          surface,
-                                          SurfaceWrapper::Type::XWayland);
-
-        auto updateSurfaceWithParentContainer = [this, wrapper, surface] {
-            if (wrapper->parentSurface())
-                wrapper->parentSurface()->removeSubSurface(wrapper);
-            if (wrapper->container())
-                wrapper->container()->removeSurface(wrapper);
-
-            auto parent = surface->parentXWaylandSurface();
-            auto parentWrapper = parent ? m_rootSurfaceContainer->getSurface(parent) : nullptr;
-            // x11 surface's parent may not associate
-            if (parentWrapper) {
-                auto container = qobject_cast<Workspace *>(parentWrapper->container());
-                Q_ASSERT(container);
-                parentWrapper->addSubSurface(wrapper);
-                container->addSurface(wrapper, parentWrapper->workspaceId());
-            } else {
-                m_workspace->addSurface(wrapper);
+    surface->safeConnect(&qw_xwayland_surface::notify_associate, this, [this, surface = QPointer<WXWaylandSurface>(surface)] {
+        auto raw = surface.data();
+        if (!raw)
+            return; // surface destroyed before callback
+        // If prelaunch wrappers exist and resolver is available, attempt async resolve; if started, remaining logic handled in callback, then return
+        if (!m_prelaunchWrappers.isEmpty() && m_appIdResolverManager) {
+            int pidfd = raw->pidFD();
+            if (pidfd >= 0) {
+                m_pendingAppIdResolveToplevels.append(raw);
+                bool started = m_appIdResolverManager->resolvePidfd(
+                    pidfd,
+                    [this, surface](const QString &appId) {
+                        auto raw = surface.data();
+                        if (!raw)
+                            return; // surface destroyed before callback
+                        int idx = m_pendingAppIdResolveToplevels.indexOf(raw);
+                        if (idx < 0)
+                            return; // removed before callback
+                        SurfaceWrapper *w = matchOrCreateXwaylandWrapper(raw, appId);
+                        initXwaylandWrapperCommon(raw, w);
+                        m_pendingAppIdResolveToplevels.removeAt(idx);
+                    });
+                if (started) {
+                    qCDebug(treelandShell)
+                        << "(XWayland) AppIdResolver request sent (callback)";
+                    return; // async path
+                } else {
+                    int idx = m_pendingAppIdResolveToplevels.indexOf(raw);
+                    if (idx >= 0)
+                        m_pendingAppIdResolveToplevels.removeAt(idx);
+                    qCDebug(treelandShell)
+                        << "(XWayland) requestResolve failed pidfd=" << pidfd;
+                }
             }
-        };
-
-        surface->safeConnect(&WXWaylandSurface::parentXWaylandSurfaceChanged,
-                             this,
-                             updateSurfaceWithParentContainer);
-        updateSurfaceWithParentContainer();
-
-        Q_ASSERT(wrapper->parentItem());
-        setupSurfaceWindowMenu(wrapper);
-        setupSurfaceActiveWatcher(wrapper);
-        Q_EMIT surfaceWrapperAdded(wrapper);
+        }
+        // Async path not taken: directly match or create (empty appId triggers fallback retrieval)
+        SurfaceWrapper *wrapper = matchOrCreateXwaylandWrapper(raw, QString());
+        initXwaylandWrapperCommon(raw, wrapper);
     });
     surface->safeConnect(&qw_xwayland_surface::notify_dissociate, this, [this, surface] {
         auto wrapper = m_rootSurfaceContainer->getSurface(surface->surface());
         qCDebug(treelandShell) << "WXWayland::notify_dissociate" << surface << wrapper;
-
+        // Cancel pending async resolve if still present. If wrapper never created, return.
+        if (!wrapper) {
+            if (!m_pendingAppIdResolveToplevels.removeOne(surface)) {
+                qCWarning(treelandShell)
+                    << "WXWayland::notify_dissociate for unknown surface" << surface;
+            }
+            return; // never created
+        }
+        // Persist XWayland window size
+        if (m_windowSizeStore && !wrapper->appId().isEmpty()) {
+            QSizeF sz = wrapper->normalGeometry().size();
+            if (!sz.isValid() || sz.isEmpty()) {
+                sz = wrapper->geometry().size();
+            }
+            const QSize s = sz.toSize();
+            if (s.isValid() && s.width() > 0 && s.height() > 0) {
+                m_windowSizeStore->saveSize(wrapper->appId(), s);
+            }
+        }
         Q_EMIT surfaceWrapperAboutToRemove(wrapper);
         m_rootSurfaceContainer->destroyForSurface(wrapper);
     });
 }
 
+SurfaceWrapper *ShellHandler::matchOrCreateXwaylandWrapper(WXWaylandSurface *surface,
+                                                           const QString &targetAppId)
+{
+    SurfaceWrapper *wrapper = nullptr;
+    if (!targetAppId.isEmpty()) {
+        for (int i = 0; i < m_prelaunchWrappers.size(); ++i) {
+            auto *candidate = m_prelaunchWrappers[i];
+            if (candidate->appId() == targetAppId) {
+                qCDebug(treelandShell) << "match prelaunch xwayland" << targetAppId;
+                m_prelaunchWrappers.removeAt(i);
+                wrapper = candidate;
+                wrapper->convertToNormalSurface(surface, SurfaceWrapper::Type::XWayland);
+                break;
+            }
+        }
+    }
+    if (!wrapper) {
+        wrapper = new SurfaceWrapper(Helper::instance()->qmlEngine(),
+                                     surface,
+                                     SurfaceWrapper::Type::XWayland,
+                                     targetAppId);
+        m_workspace->addSurface(wrapper);
+    }
+    return wrapper;
+}
+
+void ShellHandler::initXwaylandWrapperCommon(WXWaylandSurface *surface, SurfaceWrapper *wrapper)
+{
+    auto updateSurfaceWithParentContainer = [this, wrapper, surface] {
+        updateWrapperContainer(wrapper, surface->parentSurface());
+    };
+    surface->safeConnect(&WXWaylandSurface::parentSurfaceChanged,
+                         this,
+                         updateSurfaceWithParentContainer);
+    updateSurfaceWithParentContainer();
+    Q_ASSERT(wrapper->parentItem());
+    setupSurfaceWindowMenu(wrapper);
+    setupSurfaceActiveWatcher(wrapper);
+    Q_EMIT surfaceWrapperAdded(wrapper);
+}
+
 void ShellHandler::setupSurfaceActiveWatcher(SurfaceWrapper *wrapper)
 {
     Q_ASSERT_X(wrapper->container(), Q_FUNC_INFO, "Must setContainer at first!");
@@ -312,7 +531,7 @@ void ShellHandler::setupSurfaceActiveWatcher(SurfaceWrapper *wrapper)
         });
         */
     } else if (wrapper->type() == SurfaceWrapper::Type::Layer) {
-        connect(wrapper, &SurfaceWrapper::requestActive, this, [this, wrapper]() {
+        connect(wrapper, &SurfaceWrapper::requestActive, this, [wrapper]() {
             auto layerSurface = qobject_cast<WLayerSurface *>(wrapper->shellSurface());
             if (layerSurface->keyboardInteractivity() == WLayerSurface::KeyboardInteractivity::None)
                 return;
@@ -327,7 +546,7 @@ void ShellHandler::setupSurfaceActiveWatcher(SurfaceWrapper *wrapper)
                 Helper::instance()->activateSurface(wrapper);
         });
 
-        connect(wrapper, &SurfaceWrapper::requestInactive, this, [this, wrapper]() {
+        connect(wrapper, &SurfaceWrapper::requestInactive, this, [this]() {
             Helper::instance()->activateSurface(m_workspace->current()->latestActiveSurface());
         });
     } else { // Xdgtoplevel or X11
