@@ -3,7 +3,6 @@
 
 #include "treeland.h"
 
-#include "compositor1adaptor.h"
 #include "treelandconfig.hpp"
 #include "core/qmlengine.h"
 #include "greeter/usermodel.h"
@@ -12,6 +11,7 @@
 #include "seat/helper.h"
 #include "utils/cmdline.h"
 #include "common/treelandlogging.h"
+#include "common/constants.h"
 
 #include <qqml.h>
 
@@ -34,6 +34,10 @@ using namespace DDM;
 #include <wxwayland.h>
 
 #include <QCoreApplication>
+#include <QDBusAbstractAdaptor>
+#include <QDBusConnection>
+#include <QDBusConnectionInterface>
+#include <QDBusMessage>
 #include <QDebug>
 #include <QLocalSocket>
 #include <QLoggingCategory>
@@ -79,6 +83,8 @@ public:
         connect(qmlEngine, &QQmlEngine::quit, q, &Treeland::quit, Qt::QueuedConnection);
         helper = qmlEngine->singletonInstance<Helper *>("Treeland", "Helper");
         connect(helper, &Helper::requestQuit, q, &Treeland::quit, Qt::QueuedConnection);
+
+        qputenv("WLR_XWAYLAND", QByteArray(LIBEXEC_DIR) + "/treeland-xwayland");
         helper->init(q);
 
 #ifndef DISABLE_DDM
@@ -260,6 +266,50 @@ private:
     std::map<PluginInterface *, QTranslator *> pluginTs;
 };
 
+
+class Compositor1Adaptor: public QDBusAbstractAdaptor
+{
+    Q_OBJECT
+    Q_CLASSINFO("D-Bus Interface", "org.deepin.Compositor1")
+    Q_CLASSINFO("D-Bus Introspection",
+                "  <interface name=\"org.deepin.Compositor1\">\n"
+                "    <method name=\"ActivateWayland\">\n"
+                "      <arg direction=\"in\" type=\"h\" name=\"fd\"/>\n"
+                "      <arg direction=\"out\" type=\"b\" name=\"result\"/>\n"
+                "    </method>\n"
+                "    <method name=\"XWaylandName\">\n"
+                "      <arg direction=\"out\" type=\"s\" name=\"result\"/>\n"
+                "      <arg direction=\"out\" type=\"ay\" name=\"auth\"/>\n"
+                "    </method>\n"
+                "    <signal name=\"SessionChanged\"/>\n"
+                "  </interface>\n"
+                "")
+public:
+    Compositor1Adaptor(Treeland *parent)
+    : QDBusAbstractAdaptor(parent)
+    {
+        setAutoRelaySignals(true);
+    }
+
+    inline Treeland *parent() const
+    { return static_cast<Treeland *>(QObject::parent()); }
+
+public Q_SLOTS: // METHODS
+
+    bool ActivateWayland(const QDBusUnixFileDescriptor &fd)
+    {
+        return parent()->ActivateWayland(fd);
+    }
+
+    void XWaylandName()
+    {
+        parent()->XWaylandName();
+    }
+    
+Q_SIGNALS: // SIGNALS
+    void SessionChanged();
+};
+
 Treeland::Treeland()
     : QObject()
     , d_ptr(std::make_unique<TreelandPrivate>(this))
@@ -271,16 +321,16 @@ Treeland::Treeland()
     d->init();
 
     if (CmdLine::ref().run().has_value()) {
-        auto exec = [runCmd = CmdLine::ref().run().value(), this, d] {
+        auto exec = [runCmd = CmdLine::ref().run().value(), d] {
             qCInfo(treelandDBus) << "run cmd:" << runCmd;
             if (auto cmdline = CmdLine::ref().unescapeExecArgs(runCmd); cmdline) {
                 auto cmdArgs = cmdline.value();
 
                 auto envs = QProcessEnvironment::systemEnvironment();
-                envs.insert("WAYLAND_DISPLAY", d->helper->defaultWaylandSocket()->fullServerName());
+                envs.insert("WAYLAND_DISPLAY", d->helper->globalWaylandSocket()->fullServerName());
                 if (auto *xwayland = d->helper->xwaylandForUid(getuid())) {
                     envs.insert("DISPLAY", xwayland->displayName());
-                } else if (auto *current = d->helper->defaultXWaylandSocket()) {
+                } else if (auto *current = d->helper->globalXWayland()) {
                     envs.insert("DISPLAY", current->displayName());
                 }
                 envs.insert("XDG_SESSION_DESKTOP", "Treeland");
@@ -298,7 +348,7 @@ Treeland::Treeland()
         auto con =
             connect(d->helper, &Helper::socketFileChanged, this, exec, Qt::SingleShotConnection);
 
-        WSocket *defaultSocket = d->helper->defaultWaylandSocket();
+        WSocket *defaultSocket = d->helper->globalWaylandSocket();
         if (defaultSocket && defaultSocket->isValid()) {
             QObject::disconnect(con);
             exec();
@@ -379,7 +429,11 @@ bool Treeland::ActivateWayland(QDBusUnixFileDescriptor _fd)
     pw = getpwuid(uid);
     QString user{ pw->pw_name };
 
-    auto socket = std::make_shared<WSocket>(true);
+    WSocket *sessionSocket = d->helper->waylandSocketForUid(uid);
+    if (!sessionSocket)
+        return false;
+    std::shared_ptr<WSocket> socket = std::make_shared<WSocket>(sessionSocket);
+
     socket->create(fd->fileDescriptor(), false);
 
     auto userModel =
@@ -405,7 +459,7 @@ bool Treeland::ActivateWayland(QDBusUnixFileDescriptor _fd)
     return true;
 }
 
-QString Treeland::XWaylandName()
+void Treeland::XWaylandName()
 {
     Q_D(Treeland);
 
@@ -424,43 +478,20 @@ QString Treeland::XWaylandName()
         auto reply = m.createErrorReply(QDBusError::InternalError,
                                         "Failed to prepare XWayland session");
         conn.send(reply);
-        return {};
+        return;
     }
     const QString &display = xwayland->displayName();
 
-    QProcess *process = new QProcess(this);
-    connect(process, &QProcess::finished, [process, m, conn, user, display] {
-        if (process->exitCode() != 0) {
-            qCWarning(treelandDBus) << "xhost command failed with exit code" << process->exitCode()
-                               << process->readAllStandardOutput()
-                               << process->readAllStandardError();
-            auto reply =
-                m.createErrorReply(QDBusError::InternalError, "Failed to set xhost permissions");
-            conn.send(reply);
-        } else {
-            qCDebug(treelandDBus) << process->exitCode() << " " << process->readAllStandardOutput()
-                             << process->readAllStandardError();
-            qCDebug(treelandDBus) << QString("user %1 got xwayland display %2.").arg(user).arg(display);
-            auto reply = m.createReply(display);
-            conn.send(reply);
-        }
-        process->deleteLater();
-    });
+    QFile authFile(QStringLiteral("/tmp/.xauth_%1").arg(display));
+    if (!authFile.open(QIODevice::ReadOnly)) {
+        conn.send(m.createErrorReply(QDBusError::InternalError, "Failed to open xauth file"));
+        return;
+    }
 
-    connect(process, &QProcess::errorOccurred, [this, m, conn] {
-        auto reply =
-            m.createErrorReply(QDBusError::InternalError, "Failed to set xhost permissions");
-        conn.send(reply);
-    });
-
-    auto env = QProcessEnvironment::systemEnvironment();
-    env.insert("DISPLAY", display);
-    process->setProcessEnvironment(env);
-    process->setProgram("xhost");
-    process->setArguments({ QString("+si:localuser:%1").arg(user) });
-    process->start();
-
-    return {};
+    QVariantList result;
+    result << QVariant(display) << QVariant(authFile.readAll());
+    conn.send(m.createReply(result));
+    return;
 }
 
 void Treeland::quit()
