@@ -47,7 +47,7 @@ Q_SIGNALS:
 
 public Q_SLOTS:
     bool ActivateWayland(QDBusUnixFileDescriptor fd);
-    QString XWaylandName();
+    void XWaylandName();
 
 private:
     void quit();
